import torch
from dtokenizer.audio.model.encodec_model import EncodecTokenizer
import torchaudio

ht = EncodecTokenizer('encodec_24k_6bps')
code, stuff_for_decode = ht.encode_file('./sample2_22k.wav')
wav_values = ht.decode(stuff_for_decode)
torchaudio.save('output.wav', torch.from_numpy(wav_values), 22050)



import math
import os
import nlp2
import torch
from semanticodec.main import AUDIOMAE_PATCH_DURATION, SAMPLE_RATE, SEGMENT_DURATION, MEL_TARGET_LENGTH
from semanticodec.utils import extract_kaldi_fbank_feature

from dtokenizer.interface import BaseTokenizer

try:
    from semanticodec import SemantiCodec
except ImportError:
    raise ImportError(
        "Please install semanticodec: pip install git+https://github.com/haoheliu/SemantiCodec-inference.git")

CONFIG = {
    "semanticodec_25_035": (25, 32768),
    "semanticodec_25_034": (25, 16384),
    "semanticodec_25_033": (25, 8192),
    "semanticodec_25_031": (25, 4096),
}


class SemanticodecTokenizer(BaseTokenizer):
    def __init__(self, config):
        self.sampling_rate = 16000
        if config in CONFIG:
            token_rate, semantic_vocab_size = CONFIG[config]
            self.model = SemantiCodec(token_rate, semantic_vocab_size)
        else:
            raise ValueError(f"config {config} not found in {CONFIG.keys()}")

    def encode(self, speech, sampling_rate):
        if speech.shape[0] > 1:
            speech = speech[0:1]
        original_duration = speech.shape[1] / sampling_rate
        original_duration = original_duration + (
                AUDIOMAE_PATCH_DURATION - original_duration % AUDIOMAE_PATCH_DURATION
        )
        target_token_len = (
                8 * original_duration / AUDIOMAE_PATCH_DURATION / self.stack_factor_K
        )
        segment_sample_length = int(sampling_rate * SEGMENT_DURATION)
        if speech.shape[1] % segment_sample_length < segment_sample_length:
            speech = torch.cat(
                [
                    speech,
                    torch.zeros(
                        1,
                        int(
                            segment_sample_length
                            - speech.shape[1] % segment_sample_length
                        ),
                    ),
                ],
                dim=1,
            )

        mel_target_length = MEL_TARGET_LENGTH * int(
            speech.shape[1] / segment_sample_length
        )
        mel = extract_kaldi_fbank_feature(
            speech, sampling_rate, target_length=mel_target_length
        )["ta_kaldi_fbank"].unsqueeze(0)
        tokens = self.model.encoder(mel.to(self.device))
        tokens = tokens[:, : math.ceil(target_token_len), :]
        return tokens, None

    def encode_file(self, input_file):
        return self.model.encode(input_file), None

    def decode(self, code):
        return self.model.decode(code)



import os
import nlp2

from dtokenizer.interface import BaseTokenizer
from .modeling_hubert import _Speech2Code, _Code2Speech
import torchaudio


class HubertTokenizer(BaseTokenizer):
    def __init__(self, config):
        self.sampling_rate = 16000
        if config in CONFIG:
            self.sc, self.cs = CONFIG[config]()
        else:
            raise ValueError(f"config {config} not found in {CONFIG.keys()}")

    def encode(self, speech, sampling_rate):
        # if sampling_rate is not 16000
        if sampling_rate != self.sampling_rate:
            speech = torchaudio.functional.resample(speech, sampling_rate, self.sampling_rate)
        return self.sc(input_values=speech), None

    def encode_file(self, input_file):
        return self.sc(filepaths=[input_file])['code'], None

    def decode(self, code):
        if not self.cs:
            raise ValueError("No hubert vocoder is available")
        else:
            return self.cs(code)


def hubert_layer6_code50(sampling_rate=16000,
                         chunk_sec=10,
                         worker=8,
                         return_diff=False,
                         batch=None):
    nlp2.download_file(
        'https://dl.fbaipublicfiles.com/textless_nlp/gslm/hubert/km50/km.bin', './')
    if not os.path.exists('./hubert_base_ls960_L6_km50.bin'):
        os.rename('./km.bin', './hubert_base_ls960_L6_km50.bin')
    sc = _Speech2Code("facebook/hubert-base-ls960", './hubert_base_ls960_L6_km50.bin', 6,
                      sampling_rate=sampling_rate,
                      chunk_sec=chunk_sec,
                      worker=worker,
                      return_diff=return_diff,
                      batch=batch)
    return sc, None


def hubert_layer6_code100(sampling_rate=16000,
                          chunk_sec=10,
                          worker=8,
                          return_diff=False,
                          batch=None):
    # https://github.com/facebookresearch/fairseq/blob/ust/examples/speech_to_speech/docs/direct_s2st_discrete_units.md
    nlp2.download_file(
        'https://dl.fbaipublicfiles.com/textless_nlp/gslm/hubert/km100/km.bin', './')
    os.rename('./km.bin', './hubert_base_ls960_L6_km100.bin')
    nlp2.download_file(
        'https://dl.fbaipublicfiles.com/fairseq/speech_to_speech/vocoder/code_hifigan/hubert_base_100_lj/g_00500000',
        './', 'hifigan_hubert_layer6_code100_g_00500000')
    nlp2.download_file(
        'https://dl.fbaipublicfiles.com/fairseq/speech_to_speech/vocoder/code_hifigan/hubert_base_100_lj/config.json',
        './', 'hifigan_hubert_layer6_code100_config.json')
    sc = _Speech2Code("facebook/hubert-base-ls960", './hubert_base_ls960_L6_km100.bin', 6,
                      sampling_rate=sampling_rate,
                      chunk_sec=chunk_sec,
                      worker=worker,
                      return_diff=return_diff,
                      batch=batch)
    cs = _Code2Speech(tts_checkpoint='./hifigan_hubert_layer6_code100_g_00500000',
                      model_cfg=nlp2.read_json('./hifigan_hubert_layer6_code100_config.json'))
    return sc, cs


def hubert_layer6_code200(sampling_rate=16000,
                          chunk_sec=10,
                          worker=8,
                          return_diff=False,
                          batch=None):
    nlp2.download_file(
        'https://dl.fbaipublicfiles.com/textless_nlp/gslm/hubert/km200/km.bin', './')
    os.rename('./km.bin', './hubert_base_ls960_L6_km200.bin')
    sc = _Speech2Code("facebook/hubert-base-ls960", './hubert_base_ls960_L6_km200.bin', 6,
                      sampling_rate=sampling_rate,
                      chunk_sec=chunk_sec,
                      worker=worker,
                      return_diff=return_diff,
                      batch=batch)
    return sc, None


def hubert_layer9_code500(sampling_rate=16000,
                          chunk_sec=10,
                          worker=8,
                          return_diff=False,
                          batch=None):
    nlp2.download_file(
        'https://dl.fbaipublicfiles.com/hubert/hubert_base_ls960_L9_km500.bin', './')
    sc = _Speech2Code("facebook/hubert-base-ls960", './hubert_base_ls960_L9_km500.bin', 9,
                      sampling_rate=sampling_rate,
                      chunk_sec=chunk_sec,
                      worker=worker,
                      return_diff=return_diff,
                      batch=batch)
    return sc, None


def zh_hubert_layer20_code2000(sampling_rate=16000,
                               chunk_sec=10,
                               worker=8,
                               return_diff=False,
                               batch=None):
    nlp2.download_file(
        'https://huggingface.co/anthony-wss/extract-ssl-bpe/resolve/main/km_2000.mdl', './')
    sc = _Speech2Code("TencentGameMate/chinese-hubert-large", './km_2000.mdl', 20,
                      sampling_rate=sampling_rate,
                      chunk_sec=chunk_sec,
                      worker=worker,
                      return_diff=return_diff,
                      batch=batch)
    return sc, None


CONFIG = {
    "hubert_layer6_code50": hubert_layer6_code50,
    "hubert_layer6_code100": hubert_layer6_code100,
    "hubert_layer6_code200": hubert_layer6_code200,
    "hubert_layer9_code500": hubert_layer9_code500,
    "zh_hubert_layer20_code2000": zh_hubert_layer20_code2000,
}



import torch
from dtokenizer.audio.model.semanticodec_model import SemanticodecTokenizer
import torchaudio

ht = SemanticodecTokenizer('semanticodec_25_035')
code, stuff_for_decode = ht.encode_file('./sample2_22k.wav')
waveform = ht.decode(code)
print(code)
import soundfile as sf

sf.write("output.wav", waveform[0, 0], 16000)



from dtokenizer.audio.model.hubert_model import HubertTokenizer

ht = HubertTokenizer('hubert_layer6_code100')
print(ht.encode_file('./sample2_22k.wav'))
code, decodec_stuff = ht.encode_file('./sample2_22k.wav')
print(code)
wav_values = ht.decode(code)
# write wav file
import soundfile as sf

sf.write('output.wav', wav_values, 16000)



class BaseTokenizer:
    def __init__(self):
        pass

    def encode(self, input_value):
        return input_value

    def encode_file(self, input_file):
        return input_file

    def decode(self, code):
        return code

    def batch_encode(self, input_values):
        return [self.encode(input_value) for input_value in input_values]

    def batch_file_encode(self, input_values):
        return [self.encode_file(input_value) for input_value in input_values]

    def batch_decode(self, codes):
        return [self.decode(code) for code in codes]



import soundfile
import torch

from dtokenizer.interface import BaseTokenizer
from SoundCodec import codec

CONFIG = {
    "encodec_24k_1_5bps": "encodec_24k_1_5bps",
    "encodec_24k_3bps": "encodec_24k_3bps",
    "encodec_24k_6bps": "encodec_24k_6bps",
    "encodec_24k_12bps": "encodec_24k_12bps",
    "encodec_24k_24bps": "encodec_24k_24bps"
}


class EncodecTokenizer(BaseTokenizer):
    def __init__(self, config):
        self.model = codec.load_codec(config)

    def encode(self, input_array, sampling_rate):
        data_item = {'audio': {'array': input_array,
                               'sampling_rate': sampling_rate}}
        unit_item = self.model.extract_unit(data_item)
        return unit_item.unit,unit_item.stuff_for_synth

    def encode_file(self, input_file):
        # read audio file into array and sampling_rate
        input_array, sampling_rate = soundfile.read(input_file)
        data_item = {'audio': {'array': input_array,
                               'sampling_rate': sampling_rate}}
        unit_item = self.model.extract_unit(data_item)
        return unit_item.unit,unit_item.stuff_for_synth

    def decode(self, stuff_for_decode):
        return self.model.decode_unit(stuff_for_decode)



import torch


def collate_fn_pad(batch, device):
    '''
    Padds batch of variable length
    note: it converts things ToTensor manually here since the ToTensor transform
    assume it takes in images rather than arbitrary tensors.
    '''
    ## get sequence lengths
    lengths = torch.tensor([t.shape[0] for t in batch]).to(device)
    ## padd
    batch = [torch.Tensor(t).to(device) for t in batch]
    batch = torch.nn.utils.rnn.pad_sequence(batch, batch_first=True, padding_value=0)
    ## compute mask
    mask = (batch != 0).to(device)
    return batch, lengths, mask


def chunks(l, n):
    """Yield successive n-sized chunks from l."""
    for i in range(0, len(l), n):
        yield l[i:i + n]
